@@ -2,6 +2,7 @@
 /*
  * Copyright (C) 2004-2008 Imendio AB
  * Copyright (C) 2010 Lanedo GmbH
+ * Copyright (C) 2012 Thomas Bechtold <toabctl@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,12 +25,13 @@
 #include <string.h>
 #include "dh-util.h"
 #include "dh-preferences.h"
-#include "ige-conf.h"
 #include "dh-app.h"
+#include "dh-settings.h"
 
 typedef struct {
         GtkWidget     *dialog;
         DhBookManager *book_manager;
+        DhSettings    *settings;
 
         /* Fonts tab */
         GtkWidget *system_fonts_button;
@@ -47,27 +49,6 @@ typedef struct {
         GtkWidget    *bookshelf_group_by_language_button;
 } DhPreferences;
 
-/* Fonts-tab related */
-static void     preferences_fonts_font_set_cb               (GtkFontButton    *button,
-                                                             gpointer          user_data);
-static void     preferences_fonts_system_fonts_toggled_cb   (GtkToggleButton  *button,
-                                                             gpointer          user_data);
-#if 0
-static void     preferences_fonts_var_font_notify_cb        (IgeConf          *client,
-                                                             const gchar      *path,
-                                                             gpointer          user_data);
-static void     preferences_fonts_fixed_font_notify_cb      (IgeConf          *client,
-                                                             const gchar      *path,
-                                                             gpointer          user_data);
-static void     preferences_fonts_use_system_font_notify_cb (IgeConf          *client,
-                                                             const gchar      *path,
-                                                             gpointer          user_data);
-static void     preferences_connect_conf_listeners          (void);
-#endif
-static void     preferences_fonts_get_font_names            (gboolean          use_system_fonts,
-                                                             gchar           **variable,
-                                                             gchar           **fixed);
-
 /* Bookshelf-tab related */
 static void     preferences_bookshelf_tree_selection_toggled_cb    (GtkCellRendererToggle *cell_renderer,
                                                                     gchar                 *path,
@@ -96,16 +77,6 @@ static void     preferences_bookshelf_find_book                    (DhBook
 static void     preferences_bookshelf_group_by_language_cb         (GObject               *object,
                                                                     GParamSpec            *pspec,
                                                                     gpointer               user_data);
-static void     preferences_bookshelf_group_by_language_toggled_cb (GtkToggleButton       *button,
-                                                                    gpointer               user_data);
-
-#define DH_CONF_PATH                  "/apps/devhelp"
-#define DH_CONF_USE_SYSTEM_FONTS      DH_CONF_PATH "/ui/use_system_fonts"
-#define DH_CONF_VARIABLE_FONT         DH_CONF_PATH "/ui/variable_font"
-#define DH_CONF_FIXED_FONT            DH_CONF_PATH "/ui/fixed_font"
-#define DH_CONF_SYSTEM_VARIABLE_FONT  "/desktop/gnome/interface/font_name"
-#define DH_CONF_SYSTEM_FIXED_FONT     "/desktop/gnome/interface/monospace_font_name"
-#define DH_CONF_GROUP_BY_LANGUAGE     DH_CONF_PATH "/ui/use_system_fonts"
 
 /* Book list store columns... */
 #define LTCOLUMN_ENABLED      0
@@ -132,6 +103,7 @@ preferences_init (void)
         }
 
         prefs = g_new0 (DhPreferences, 1);
+        prefs->settings = dh_settings_get ();
         prefs->book_manager = g_object_ref (dh_app_peek_book_manager (DH_APP (app)));
         g_signal_connect (prefs->book_manager,
                           "book-created",
@@ -154,7 +126,8 @@ preferences_shutdown (void)
                 return;
         }
 
-        g_object_unref (prefs->book_manager);
+        g_clear_object (&prefs->settings);
+        g_clear_object (&prefs->book_manager);
         gtk_list_store_clear (prefs->bookshelf_store);
         gtk_widget_destroy (GTK_WIDGET (prefs->dialog));
 
@@ -162,25 +135,6 @@ preferences_shutdown (void)
         prefs = NULL;
 }
 
-static void
-preferences_fonts_font_set_cb (GtkFontButton *button,
-                               gpointer       user_data)
-{
-	DhPreferences *prefs = user_data;
-	const gchar   *font_name;
-	const gchar   *key;
-
-	font_name = gtk_font_button_get_font_name (button);
-
-	if (GTK_WIDGET (button) == prefs->variable_font_button) {
-		key = DH_CONF_VARIABLE_FONT;
-	} else {
-		key = DH_CONF_FIXED_FONT;
-	}
-
-	ige_conf_set_string (ige_conf_get (), key, font_name);
-}
-
 static void
 preferences_fonts_system_fonts_toggled_cb (GtkToggleButton *button,
                                            gpointer         user_data)
@@ -190,131 +144,9 @@ preferences_fonts_system_fonts_toggled_cb (GtkToggleButton *button,
 
 	active = gtk_toggle_button_get_active (button);
 
-	ige_conf_set_bool (ige_conf_get (),
-                           DH_CONF_USE_SYSTEM_FONTS,
-                           active);
-
 	gtk_widget_set_sensitive (prefs->fonts_table, !active);
 }
 
-#if 0
-static void
-preferences_fonts_var_font_notify_cb (IgeConf     *client,
-                                      const gchar *path,
-                                      gpointer     user_data)
-{
-	DhPreferences *prefs = user_data;
-	gboolean       use_system_fonts;
-	gchar         *font_name;
-
-        ige_conf_get_bool (ige_conf_get (),
-                           DH_CONF_USE_SYSTEM_FONTS,
-                           &use_system_fonts);
-
-	if (prefs->variable_font_button) {
-		ige_conf_get_string (ige_conf_get (), path, &font_name);
-		gtk_font_button_set_font_name (GTK_FONT_BUTTON (prefs->variable_font_button),
-					       font_name);
-                g_free (font_name);
-	}
-}
-
-static void
-preferences_fonts_fixed_font_notify_cb (IgeConf     *client,
-                                        const gchar *path,
-                                        gpointer     user_data)
-{
-	DhPreferences *prefs = user_data;
-	gboolean       use_system_fonts;
-	gchar         *font_name;
-
-	ige_conf_get_bool (ige_conf_get (),
-                           DH_CONF_USE_SYSTEM_FONTS,
-                           &use_system_fonts);
-
-	if (prefs->fixed_font_button) {
-                ige_conf_get_string (ige_conf_get (), path, &font_name);
-		gtk_font_button_set_font_name (GTK_FONT_BUTTON (prefs->fixed_font_button),
-					       font_name);
-                g_free (font_name);
-	}
-}
-
-static void
-preferences_fonts_use_system_font_notify_cb (IgeConf     *client,
-                                             const gchar *path,
-                                             gpointer     user_data)
-{
-	DhPreferences *prefs = user_data;
-	gboolean       use_system_fonts;
-
-	ige_conf_get_bool (ige_conf_get (), path, &use_system_fonts);
-
-	if (prefs->system_fonts_button) {
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->system_fonts_button),
-					      use_system_fonts);
-	}
-
-	if (prefs->fonts_table) {
-		gtk_widget_set_sensitive (prefs->fonts_table, !use_system_fonts);
-	}
-}
-
-/* FIXME: This is not hooked up yet (to update the dialog if the values are
- * changed outside of devhelp).
- */
-static void
-preferences_connect_conf_listeners (void)
-{
-	IgeConf *conf;
-
-	conf = ige_conf_get ();
-
-	prefs->use_system_fonts_id =
-		ige_conf_notify_add (conf,
-                                     DH_CONF_USE_SYSTEM_FONTS,
-                                     preferences_use_system_font_notify_cb,
-                                     prefs);
-	prefs->system_var_id =
-		ige_conf_notify_add (conf,
-                                     DH_CONF_SYSTEM_VARIABLE_FONT,
-                                     preferences_var_font_notify_cb,
-                                     prefs);
-	prefs->system_fixed_id =
-		ige_conf_notify_add (conf,
-                                     DH_CONF_SYSTEM_FIXED_FONT,
-                                     preferences_fixed_font_notify_cb,
-                                     prefs);
-	prefs->var_id =
-		ige_conf_notify_add (conf,
-                                     DH_CONF_VARIABLE_FONT,
-                                     preferences_var_font_notify_cb,
-                                     prefs);
-	prefs->fixed_id =
-		ige_conf_notify_add (conf,
-                                     DH_CONF_FIXED_FONT,
-                                     preferences_fixed_font_notify_cb,
-                                     prefs);
-}
-#endif
-
-static void
-preferences_fonts_get_font_names (gboolean   use_system_fonts,
-                                  gchar    **variable,
-                                  gchar    **fixed)
-{
-	gchar   *name;
-	gdouble  size;
-
-	dh_util_font_get_variable (&name, &size, use_system_fonts);
-	*variable = g_strdup_printf ("%s %u", name, (guint)size);
-	g_free (name);
-
-	dh_util_font_get_fixed (&name, &size, use_system_fonts);
-	*fixed = g_strdup_printf ("%s %u", name, (guint)size);
-	g_free (name);
-}
-
 static void
 preferences_bookshelf_set_language_inconsistent (const gchar *language)
 {
@@ -800,28 +632,12 @@ preferences_bookshelf_clean_store (void)
         gtk_list_store_clear (prefs->bookshelf_store);
 }
 
-static void
-preferences_bookshelf_group_by_language_toggled_cb (GtkToggleButton *button,
-                                                    gpointer         user_data)
-{
-	DhPreferences *prefs = user_data;
-	gboolean       active;
-
-	active = gtk_toggle_button_get_active (button);
-
-        if (dh_book_manager_get_group_by_language (prefs->book_manager) != active) {
-                dh_book_manager_set_group_by_language (prefs->book_manager,
-                                                       active);
-        }
-}
-
 void
 dh_preferences_show_dialog (void)
 {
         gchar      *path;
 	GtkBuilder *builder;
 	gboolean    use_system_fonts;
-	gchar      *var_font_name, *fixed_font_name;
 
         preferences_init ();
 
@@ -847,36 +663,33 @@ dh_preferences_show_dialog (void)
                 NULL);
         g_free (path);
 
+        /* setup GSettings bindings */
+        GSettings *settings_fonts = dh_settings_peek_fonts_settings (prefs->settings);
+        GSettings *settings_contents = dh_settings_peek_contents_settings (prefs->settings);
+        g_settings_bind (settings_fonts, "use-system-fonts",
+                         G_OBJECT (prefs->system_fonts_button),
+                         "active", G_SETTINGS_BIND_DEFAULT);
+        g_settings_bind (settings_fonts, "fixed-font",
+                         G_OBJECT (prefs->fixed_font_button),
+                         "font-name", G_SETTINGS_BIND_DEFAULT);
+        g_settings_bind (settings_fonts, "variable-font",
+                         G_OBJECT (prefs->variable_font_button),
+                         "font-name", G_SETTINGS_BIND_DEFAULT);
+
+        g_settings_bind (settings_contents,
+                         "group-books-by-language", G_OBJECT (prefs->bookshelf_group_by_language_button),
+                         "active", G_SETTINGS_BIND_DEFAULT);
+
 	dh_util_builder_connect (
                 builder,
                 prefs,
-                "variable_font_button", "font_set", preferences_fonts_font_set_cb,
-                "fixed_font_button", "font_set", preferences_fonts_font_set_cb,
                 "system_fonts_button", "toggled", preferences_fonts_system_fonts_toggled_cb,
                 "bookshelf_enabled_toggle", "toggled", preferences_bookshelf_tree_selection_toggled_cb,
-                "bookshelf_group_by_language_button", "toggled", preferences_bookshelf_group_by_language_toggled_cb,
                 NULL);
 
-	ige_conf_get_bool (ige_conf_get (),
-                           DH_CONF_USE_SYSTEM_FONTS,
-                           &use_system_fonts);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->system_fonts_button),
-				      use_system_fonts);
-	gtk_widget_set_sensitive (prefs->fonts_table, !use_system_fonts);
-
-	preferences_fonts_get_font_names (FALSE, &var_font_name, &fixed_font_name);
-
-	if (var_font_name) {
-		gtk_font_button_set_font_name (GTK_FONT_BUTTON (prefs->variable_font_button),
-					       var_font_name);
-		g_free (var_font_name);
-	}
-
-	if (fixed_font_name) {
-		gtk_font_button_set_font_name (GTK_FONT_BUTTON (prefs->fixed_font_button),
-					       fixed_font_name);
-		g_free (fixed_font_name);
-	}
+	/* set initial sensitive */
+        use_system_fonts = g_settings_get_boolean (settings_fonts, "use-system-fonts");
+        gtk_widget_set_sensitive (prefs->fonts_table, !use_system_fonts);
 
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->bookshelf_group_by_language_button),
                                       dh_book_manager_get_group_by_language (prefs->book_manager));
